# axios二次封装库
- 此库扩展了`axios`，提供了一些高级功能，如请求复用，请求取消， 出错重试，并发控制，请求缓存等功能
- 还预定义了一组拦截器，里面提供了一些常用的业务处理逻辑
- 该库使用ts编写，提供了类型定义和导出
- 采用了`axios`的实例模式，并导出了这个实例`instance`，如果想添加自定义拦截器，可以通过`instance.interceptors.request.use`和`instance.interceptors.response.use`

## git地址
https://git.qmpoa.com/fe_common_lib/axios-enhance

## 安装
``` json
"dependencies": {
  "axios-enhance": "git+https://git.qmpoa.com/fe_common_lib/axios-enhance.git#v1.0.0"
}
```
```
yarn install
```
> 尽量安装最新版本，最新版本就是最高的tag号

## 使用
``` javascript
import { initConfig, requestSimplified } from 'axios-enhance'

const { get: getRequest, post: postRequest } = requestSimplified

initConfig({
  config: {
    baseURL: 'http://localhost:3000/'
  },
  // 其他配置项参考文档或类型声明
})
```

## 关于参数
`get`和`post`请求的参数都放到配置项`data`上即可，在实际发送get请求前会自动赋值到`axios`的`params`上，
这么做是为了尽量保证api的一致性，减小心智负担

## 关于返回结构
提供了两组发送请求的模块，`request`和`request-simplified`，这两者导出的方法都是一样的，
都是一个`get`方法，一个`post`方法，区别在于返回结构的处理:
- `request`里的方法的返回值是完整的axios返回结构，获取数据需要`res.data.data`，因为`res.data`上挂载了很多axios用到的属性
- `request-simplified`里的方法的返回值是精简后的结构，去掉了axios那一层，获取数据只需要`res.data`

之所以提供两组，是为了兼容多个项目的写法，但实际使用看来还是精简版的更好用一些


## 请求取消错误处理
接口被取消时，会抛出一个reject的Promise，有时可能不想看到这个错误，
可通过`initConfig`函数的第二个参数来配置如何处理

| 值        | 说明                                                                                                    |
|----------|-------------------------------------------------------------------------------------------------------|
| error    | 不进行处理，会在错误对象上挂载被取消的接口以及新的接口的配置                                                                        |                           |
| log      | 默认值，表示不抛出错误，只在控制台打印一下被取消的接口及参数![img.png](doc_img/img.png)                                                     |
| ignore   | 直接忽略，既不抛出错误，也不打印                                                                                      |
| Function | `(e: PromiseRejectionEvent, oldConfig: ApiConfig, newConfig: ApiConfig \| undefined) => void`<br/> 当调用导出的`cancelRequest`方法时，newConfig会为`undefined` |

## TODO
- 自定义判断请求重试 ✅
- 缓存数据支持存储到storage ✅
- 支持设置`showErrMsg`配置 ✅
- 支持处理`bigInt`返回数据 ✅
- 测试：`bigInt`解析（解析完了会不会对其他返回情况有影响），自定义判断请求重试，showErrMsg
- 完善错误处理 ✅
- 优化data和params ✅
- 增加默认时间格式化配置 ✅
- 完善请求重试逻辑，支持请求成功时也能重试 ✅
- 优化默认配置存储和合并逻辑 ✅
