/**
 * 请求取消
 */

import { getUrlFlag, parseUrlFlag } from './util'
import { pendingList } from './pending'
import { defaultConfig } from './default-config'
import type { ApiConfig, PendingItem } from './type'

export const executeCanceler = (config: ApiConfig) => {
  const urlFlag = getUrlFlag(config)
  pendingList.forEach(item => {
    if (item.canceler && item.urlFlag === urlFlag) {
      const { cancel, payload } = config
      let shouldCancel: boolean = false
      if (typeof cancel === 'boolean') {
        shouldCancel = cancel
      } else if (typeof cancel === 'function') {
        shouldCancel = cancel(payload, item.payload)
      } else if (typeof cancel === 'string') {
        shouldCancel = payload[cancel] === item.payload[cancel]
      } else if (Array.isArray(cancel)) {
        shouldCancel = cancel.every(key => payload[key] === item.payload[key])
      }
      shouldCancel && item.canceler(config) // 对该接口之前发送的的同名接口执行取消操作
    }
  })
}

export const isCancelError = (error: any): boolean => !!error?.message?.startsWith(defaultConfig.cancelMessage)

type CancelFilter = (item: Pick<PendingItem, 'urlFlag' | 'payload'> & { parsedUrlFlag: ReturnType<typeof parseUrlFlag> }) => boolean
export const cancelRequest = (filterPendingItem: CancelFilter) => {
  // 需要先过滤出来再依次执行，因为执行的时候会动态删除，可能会导致漏执行
  pendingList
    .filter(item => {
      return item.canceler && filterPendingItem({
        urlFlag: item.urlFlag,
        payload: item.payload,
        parsedUrlFlag: parseUrlFlag(item.urlFlag)
      })
    })
    .forEach(item => {
      item.canceler()
    })
}
