/**
 * 并发控制
 */

import { defaultConfig } from './default-config'
import { pendingList } from './pending'
import { ApiConfig } from './type'

interface Concurrency {
  timestamp: number // 加入并发队列的时间
  handler: (...args: any[]) => void
}

// 总的并发控制
const concurrency: Concurrency[] = []

// 针对某个接口的并发控制
const concurrencyMap: { [urlFlag: string]: Concurrency[] } = {}

export const pushConcurrency = (handler: Concurrency['handler'], urlFlag?: string) => {
  const item: Concurrency = { timestamp: Date.now(), handler }
  if (urlFlag) {
    concurrencyMap[urlFlag] = [...(concurrencyMap[urlFlag] || []), item]
  } else {
    concurrency.push(item)
  }
}

export const dischargeConcurrency = (urlFlag: string) => {
  const arr = [concurrency.shift(), concurrencyMap[urlFlag]?.shift()].filter(Boolean)
  if (!arr.length) return
  if (arr.length > 1) {
    // 取出最早加入并发队列的请求执行
    arr.sort((a, b) => a.timestamp - b.timestamp)
  }
  arr[0].handler()
}

export const getConcurrencyPromise = (config: ApiConfig, urlFlag: string): Promise<void> | undefined => {
  if (defaultConfig.concurrency && pendingList.length >= defaultConfig.concurrency) {
    return new Promise(resolve => {
      pushConcurrency(resolve)
    })
  }
  if (typeof config.cancel === 'number') {
    const concurrencyCount = config.cancel
    const pendingItems = pendingList.filter(e => e.urlFlag === urlFlag)
    if (pendingItems.length >= concurrencyCount) {
      return new Promise(resolve => {
        pushConcurrency(resolve, urlFlag)
      })
    }
  }
}
