import axios from 'axios'
import { defaultConfig } from './default-config'
import { getUrlFlag } from './util'
import type { ApiConfig, PendingItem } from './type'

const CancelToken = axios.CancelToken

// 已发送但还未返回的请求列表
export const pendingList: PendingItem[] = []

export const pushPending = (config: ApiConfig): PendingItem => {
  const urlFlag = getUrlFlag(config) // url标记
  const pendingItem: PendingItem = { urlFlag, payload: config.payload || {}, reqId: config.reqId }
  config.cancelToken = new CancelToken(canceler => {
    pendingItem.canceler = (newConfig) => {
      canceler(`${defaultConfig.cancelMessage}：${urlFlag}`, config, newConfig)
      removePending(pendingItem.reqId)
    }
  })
  pendingList.push(pendingItem)
  return pendingItem
}

export const removePending = (reqId: string): PendingItem | undefined => {
  const index = pendingList.findIndex(item => item.reqId === reqId)
  if (index !== -1) {
    return pendingList.splice(index, 1)[0]
  }
}

/**
 * 测试`pendingList`维护的是否有问题
 * build时会被tree-shaking掉
 */
if (import.meta.env.VITE_APP_MODE === 'development') {
  setInterval(() => {
    console.log('pendingLength：', pendingList.length)
  }, 800)
}
