import { AxiosError } from 'axios'
import type { ApiConfig, ServerRes } from './type'
import { ExtendedAxiosError } from './extend-axios-error'

export const getRetryDelay = (config: ApiConfig) => {
  return typeof config.retryDelay === 'function'
    ? config.retryDelay(config.currRetryCount)
    : config.retryDelay
}

export const updateRetryConfig = (res: ServerRes | ExtendedAxiosError, config: ApiConfig): boolean => {
  config.isFinalResponse = true
  const currRetryCount = config.currRetryCount || 0
  if (!config.maxRetryCount || currRetryCount < config.maxRetryCount) {
    const shouldRetry: boolean = res instanceof AxiosError ? config.retryWhenFail(res) : config.retryWhenSuccess(res)
    if (shouldRetry) {
      config.currRetryCount = currRetryCount + 1
      config.isFinalResponse = false
    }
  }
  return config.isFinalResponse
}
