/**
 * 请求复用
 */
import { getUrlFlag } from './util'
import { isEqual } from 'lodash'
import { pendingList } from './pending'
import type { ApiConfig } from './type'

export const getEqualPayloadPromise = <R>(config: ApiConfig): Promise<R> | undefined => {
  const urlFlag = getUrlFlag(config)
  const pendingItems = pendingList.filter(item => item.urlFlag === urlFlag)
  for (let i = 0, len = pendingItems.length; i < len; i++) {
    const item = pendingItems[i]
    if (item.promise && isEqual(item.payload, config.payload)) {
      return item.promise.then(res => {
        // 接口请求结果复用，在某一个then回调里可能会修改res里的数据，因此这里传入数据的副本
        return JSON.parse(JSON.stringify(res))
      })
    }
  }
}
