//// @ts-nocheck
import { initConfig, requestSimplified } from '../src/index'

const { get: getRequest, post: postRequest } = requestSimplified

initConfig({
  config: {
    baseURL: 'http://localhost:3000/test-cache/',
    data: {
      name: 'dgf'
    },
    timeout: 2000,
    timeoutErrorMessage: '超时了'
  },
  finalConfig(config) {
    return {
      data: {
        age: 30
      },
      // cancel: 2
    }
  },
  // concurrency: 1
})

const testPost = (config = {}) => {
  return postRequest({
    url: '/post1',
    cache: true,
    successHandler: (res) => {
      // console.log('post successHandler', res)
    },
    ...config
  })
}
const testGet = (data: { sex: number; school: string }, config = {}) => {
  return getRequest<{ name: string; data: { name: string; age: number } }, typeof data>({
    url: '/get?a=1&b=2',
    maxRetryCount: 2,
    cache: {
      cache: 'localStorage',
      expireSeconds: 8
    },
    successHandler: (res) => {
      console.log('get successHandler')
    },
    successMsg: (payload) => '1111',
    handleDateField: 'YYYY-MM-DD HH:mm',
    data,
    ...config
  })
}
testGet({ sex: 1, school: '北大' })
  .then(res => {
    console.log(res.data)
    return testGet({ sex: 1, school: '北大' }).then(res => {
      console.log(res.data)
      return res
    })
  })
  .then(() => {
    setTimeout(() => {
      return testGet({ sex: 1, school: '北大' }, { cache: {
          cache: 'sessionStorage',
          expireSeconds: 10
        } }).then(res => {
        console.log(res.data)
        return res
      })
    }, 10000)
  })

// testPost({
//   data: { name: '111', age: 12 }
// })
// testPost({
//   data: { name: '111', age: 20 }
// })
// testPost({
//   data: { name: '222', age: 24 }
// })
// testPost({
//   data: { age: 24 }
// })
// testPost({
//   data: { sex: '男' }
// })
// testGet(
//   {
//     sex: 1,
//     school: '北京大学'
//   },
//   {
//     data: { page: 1, num: 20 }
//   }
// )

testPost({
  data: { name: '111' }
}).then((res) => {
  console.log('new data', res.data)
  return testPost({
    data: { name: '111' }
  })
}).then((res) => {
  console.log('cache data', res.data)
  return testPost({
    data: { sex: '男' },
    cache: 'sessionStorage'
  })
}).then((res) => {
  console.log('new data', res.data)
  return testPost({
    data: { sex: '男' },
    cache: 'localStorage'
  })
}).then(res => {
  console.log('cache data', res.data)
}).catch(e => {
  console.log('catch post', e)
})

const testBigIntRes = () => {
  return postRequest({
    url: 'https://teststartupserviceapi.qmplink.com/startup/api/account/list',
    handleBigint: true,
    headers: {
      Authorization: 'eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3MzM2NDY1MjcsInNyYyI6InN0YXJ0dXAiLCJ1dWlkIjoiODczMDFhMTFhZmFhYzgzYzQ1MGU2YjNlM2I1NGMxOTNkYjgwMDkyZWYzMmI5OTlmNWFmOTAzNGIwNGFiZTAwMSIsImNsaWVudCI6NCwiY3JlYXRlX3RpbWUiOiIyMDI0LTA5LTA5IDE2OjI4OjQ3IiwidmVyc2lvbiI6IiIsIndheSI6MCwic3JjX3V1aWQiOiIxODkxMTI0NDE3MiJ9.kFzS-TrCA15k1LaeD66N7uAx_9zuh9J91pm21JPheaQ'
    }
  })
}
testBigIntRes().then(res => {
  console.log('bigIntFinalRes', res)
})
